@@ -4,14 +4,45 @@ Numba-specific errors and warnings.
 
 from __future__ import print_function, division, absolute_import
 
+import sys
 import contextlib
 from collections import defaultdict
 import warnings
+from numba import six
+from functools import wraps
 
 
 # Filled at the end
 __all__ = []
 
+def deprecated(arg):
+    """Define a deprecation decorator.
+    An optional string should refer to the new API to be used instead.
+
+    Example:
+      @deprecated
+      def old_func(): ...
+
+      @deprecated('new_func')
+      def old_func(): ..."""
+
+    subst = arg if isinstance(arg, str) else None
+    def decorator(func):
+        def wrapper(*args, **kwargs):
+            msg = "Call to deprecated function \"{}\"."
+            if subst:
+                msg += "\n Use \"{}\" instead."
+            warnings.warn(msg.format(func.__name__, subst),
+                          category=DeprecationWarning, stacklevel=2)
+            return func(*args, **kwargs)
+
+        return wraps(func)(wrapper)
+
+    if not subst:
+        return decorator(arg)
+    else:
+        return decorator
+
 
 class NumbaWarning(Warning):
     """
@@ -69,7 +100,29 @@ class WarningsFixer(object):
 
 
 class NumbaError(Exception):
-    pass
+    @property
+    def contexts(self):
+        try:
+            return self._contexts
+        except AttributeError:
+            self._contexts = lst = []
+            return lst
+
+    def add_context(self, msg):
+        """
+        Add contextual info.  The execption message is expanded with the new
+        contextual info.
+        """
+        self.contexts.append(msg)
+        newmsg = '{0}\n[{1}] During: {2}'.format(self, len(self.contexts), msg)
+        self.args = (newmsg,)
+        return self
+
+    def patch_message(self, new_message):
+        """
+        Change the error message to the given new message.
+        """
+        self.args = (new_message,) + self.args[1:]
 
 
 class IRError(NumbaError):
@@ -159,6 +212,43 @@ class ConstantInferenceError(NumbaError):
     """
 
 
+class InternalError(NumbaError):
+    """
+    For wrapping internal error occured within the compiler
+    """
+    def __init__(self, exception):
+        super(InternalError, self).__init__(str(exception))
+        self.old_exception = exception
+
+
+def _format_msg(fmt, args, kwargs):
+    return fmt.format(*args, **kwargs)
+
+
+@contextlib.contextmanager
+def new_error_context(fmt_, *args, **kwargs):
+    """
+    A contextmanager that prepend contextual information to any exception
+    raised within.  If the exception type is not an instance of NumbaError,
+    it will be wrapped into a InternalError.   The exception class can be
+    changed by providing a "errcls_" keyword argument with the exception
+    constructor.
+
+    The first argument is a message that describes the context.  It can be a
+    format string.  If there are additional arguments, it will be used as
+    ``fmt_.format(*args, **kwargs)`` to produce the final message string.
+    """
+    errcls = kwargs.pop('errcls_', InternalError)
+    try:
+        yield
+    except NumbaError as e:
+        e.add_context(_format_msg(fmt_, args, kwargs))
+        raise
+    except Exception as e:
+        newerr = errcls(e).add_context(_format_msg(fmt_, args, kwargs))
+        six.reraise(type(newerr), newerr, sys.exc_info()[2])
+
+
 __all__ += [name for (name, value) in globals().items()
             if not name.startswith('_') and isinstance(value, type)
                and issubclass(value, (Exception, Warning))]
