@@ -59,8 +59,8 @@ def _mangle_attr(name):
 # Class object
 
 _ctor_template = """
-def ctor({args}):
-    return __numba_cls_({args})
+def ctor(*args):
+    return __numba_cls_(*args)
 """
 
 
@@ -80,14 +80,11 @@ class JitClassType(type):
         return outcls
 
     def _set_init(cls):
-        # make ctor
-        init = cls.class_type.instance_type.methods['__init__']
-        argspec = inspect.getargspec(init)
-        assert not argspec.varargs, 'varargs not supported'
-        assert not argspec.keywords, 'keywords not supported'
-        assert not argspec.defaults, 'defaults not supported'
-        args = ', '.join(argspec.args[1:])
-        ctor_source = _ctor_template.format(args=args)
+        """
+        Generate a wrapper for calling the constructor from pure Python.
+        Note the wrapper will only accept positional arguments.
+        """
+        ctor_source = _ctor_template
         glbls = {"__numba_cls_": cls}
         exec_(ctor_source, glbls)
         ctor = glbls['ctor']
@@ -369,10 +366,10 @@ def attr_impl(context, builder, sig, args, attr):
 
         # Store n
         setattr(data, _mangle_attr(attr), val)
-        context.nrt_incref(builder, attr_type, val)
+        context.nrt.incref(builder, attr_type, val)
 
         # Delete old value
-        context.nrt_decref(builder, attr_type, oldvalue)
+        context.nrt.decref(builder, attr_type, oldvalue)
 
     elif attr in typ.jitprops:
         # It's a jitted property
@@ -406,7 +403,7 @@ def imp_dtor(context, module, instance_type):
         ptr = builder.bitcast(dtor_fn.args[0], alloc_type.as_pointer())
         data = context.make_helper(builder, alloc_fe_type, ref=ptr)
 
-        context.nrt_decref(builder, alloc_fe_type, data._getvalue())
+        context.nrt.decref(builder, alloc_fe_type, data._getvalue())
 
         builder.ret_void()
 
@@ -423,12 +420,12 @@ def ctor_impl(context, builder, sig, args):
     alloc_type = context.get_data_type(inst_typ.get_data_type())
     alloc_size = context.get_abi_sizeof(alloc_type)
 
-    meminfo = context.nrt_meminfo_alloc_dtor(
+    meminfo = context.nrt.meminfo_alloc_dtor(
         builder,
         context.get_constant(types.uintp, alloc_size),
         imp_dtor(context, builder.module, inst_typ),
     )
-    data_pointer = context.nrt_meminfo_data(builder, meminfo)
+    data_pointer = context.nrt.meminfo_data(builder, meminfo)
     data_pointer = builder.bitcast(data_pointer,
                                    alloc_type.as_pointer())
 
