@@ -147,6 +147,7 @@ class OverloadSelector(object):
         self.versions.append((sig, value))
         self._cache.clear()
 
+
 @utils.runonce
 def _load_global_helpers():
     """
@@ -155,13 +156,16 @@ def _load_global_helpers():
     # This is Py_None's real C name
     ll.add_symbol("_Py_NoneStruct", id(None))
 
-    # Add C helper functions
+    # Add Numba C helper functions
     for c_helpers in (_helperlib.c_helpers, _dynfunc.c_helpers):
-        for py_name in c_helpers:
+        for py_name, c_address in c_helpers.items():
             c_name = "numba_" + py_name
-            c_address = c_helpers[py_name]
             ll.add_symbol(c_name, c_address)
 
+    # Add Numpy C helpers (npy_XXX)
+    for c_name, c_address in _helperlib.npymath_exports.items():
+        ll.add_symbol(c_name, c_address)
+
     # Add all built-in exception classes
     for obj in utils.builtins.__dict__.values():
         if isinstance(obj, type) and issubclass(obj, BaseException):
@@ -265,6 +269,11 @@ class BaseContext(object):
     def target_data(self):
         raise NotImplementedError
 
+    @utils.cached_property
+    def nrt(self):
+        from numba.runtime.context import NRTContext
+        return NRTContext(self, self.enable_nrt)
+
     def subtarget(self, **kws):
         obj = copy.copy(self)  # shallow copy
         for k, v in kws.items():
@@ -346,7 +355,6 @@ class BaseContext(object):
     def declare_function(self, module, fndesc):
         fnty = self.call_conv.get_function_type(fndesc.restype, fndesc.argtypes)
         fn = module.get_or_insert_function(fnty, name=fndesc.mangled_name)
-        assert fn.is_declaration
         self.call_conv.decorate_function(fn, fndesc.args, fndesc.argtypes)
         if fndesc.inline:
             fn.attributes.add('alwaysinline')
@@ -867,9 +875,13 @@ class BaseContext(object):
         shapevals = [self.get_constant(types.intp, s) for s in ary.shape]
         cshape = Constant.array(llintp, shapevals)
 
-        # Handle strides: use strides of the equivalent C-contiguous array.
-        contig = np.ascontiguousarray(ary)
-        stridevals = [self.get_constant(types.intp, s) for s in contig.strides]
+        # Handle strides
+        if ary.ndim > 0:
+            # Use strides of the equivalent C-contiguous array.
+            contig = np.ascontiguousarray(ary)
+            stridevals = [self.get_constant(types.intp, s) for s in contig.strides]
+        else:
+            stridevals = []
         cstrides = Constant.array(llintp, stridevals)
 
         # Create array structure
@@ -905,6 +917,13 @@ class BaseContext(object):
         assert isinstance(ty, llvmir.Type), "Expected LLVM type"
         return ty.get_abi_alignment(self.target_data)
 
+    def get_preferred_array_alignment(context, ty):
+        """
+        Get preferred array alignment for Numba type *ty*.
+        """
+        # AVX prefers 32-byte alignment
+        return 32
+
     def post_lowering(self, mod, library):
         """Run target specific post-lowering transformation here.
         """
@@ -914,179 +933,6 @@ class BaseContext(object):
         """
         return lc.Module(name)
 
-    def _require_nrt(self):
-        if not self.enable_nrt:
-            raise RuntimeError("Require NRT")
-
-    def nrt_meminfo_alloc(self, builder, size):
-        """
-        Allocate a new MemInfo with a data payload of `size` bytes.
-
-        A pointer to the MemInfo is returned.
-        """
-        self._require_nrt()
-
-        mod = builder.module
-        fnty = llvmir.FunctionType(void_ptr,
-                                   [self.get_value_type(types.intp)])
-        fn = mod.get_or_insert_function(fnty, name="NRT_MemInfo_alloc_safe")
-        fn.return_value.add_attribute("noalias")
-        return builder.call(fn, [size])
-
-    def nrt_meminfo_alloc_dtor(self, builder, size, dtor):
-        self._require_nrt()
-
-        mod = builder.module
-        ll_void_ptr = self.get_value_type(types.voidptr)
-        fnty = llvmir.FunctionType(llvmir.IntType(8).as_pointer(),
-                                   [self.get_value_type(types.intp),
-                                    ll_void_ptr])
-        fn = mod.get_or_insert_function(fnty,
-                                        name="NRT_MemInfo_alloc_dtor_safe")
-        fn.return_value.add_attribute("noalias")
-        return builder.call(fn, [size, builder.bitcast(dtor, ll_void_ptr)])
-
-    def nrt_meminfo_alloc_aligned(self, builder, size, align):
-        """
-        Allocate a new MemInfo with an aligned data payload of `size` bytes.
-        The data pointer is aligned to `align` bytes.  `align` can be either
-        a Python int or a LLVM uint32 value.
-
-        A pointer to the MemInfo is returned.
-        """
-        self._require_nrt()
-
-        mod = builder.module
-        intp = self.get_value_type(types.intp)
-        u32 = self.get_value_type(types.uint32)
-        fnty = llvmir.FunctionType(void_ptr, [intp, u32])
-        fn = mod.get_or_insert_function(fnty,
-                                        name="NRT_MemInfo_alloc_safe_aligned")
-        fn.return_value.add_attribute("noalias")
-        if isinstance(align, int):
-            align = self.get_constant(types.uint32, align)
-        else:
-            assert align.type == u32, "align must be a uint32"
-        return builder.call(fn, [size, align])
-
-    def nrt_meminfo_new_varsize(self, builder, size):
-        """
-        Allocate a MemInfo pointing to a variable-sized data area.  The area
-        is separately allocated (i.e. two allocations are made) so that
-        re-allocating it doesn't change the MemInfo's address.
-
-        A pointer to the MemInfo is returned.
-        """
-        self._require_nrt()
-
-        mod = builder.module
-        fnty = llvmir.FunctionType(void_ptr,
-                                   [self.get_value_type(types.intp)])
-        fn = mod.get_or_insert_function(fnty, name="NRT_MemInfo_new_varsize")
-        fn.return_value.add_attribute("noalias")
-        return builder.call(fn, [size])
-
-    def nrt_meminfo_varsize_alloc(self, builder, meminfo, size):
-        """
-        Allocate a new data area for a MemInfo created by nrt_meminfo_new_varsize().
-        The new data pointer is returned, for convenience.
-
-        Contrary to realloc(), this always allocates a new area and doesn't
-        copy the old data.  This is useful if resizing a container needs
-        more than simply copying the data area (e.g. for hash tables).
-
-        The old pointer will have to be freed with nrt_meminfo_varsize_free().
-        """
-        return self._call_nrt_varsize_alloc(builder, meminfo, size,
-                                            "NRT_MemInfo_varsize_alloc")
-
-    def nrt_meminfo_varsize_realloc(self, builder, meminfo, size):
-        """
-        Reallocate a data area allocated by nrt_meminfo_new_varsize().
-        The new data pointer is returned, for convenience.
-        """
-        return self._call_nrt_varsize_alloc(builder, meminfo, size,
-                                            "NRT_MemInfo_varsize_realloc")
-
-    def nrt_meminfo_varsize_free(self, builder, meminfo, ptr):
-        """
-        Free a memory area allocated for a NRT varsize object.
-        Note this does *not* free the NRT object itself!
-        """
-        self._require_nrt()
-
-        mod = builder.module
-        fnty = llvmir.FunctionType(llvmir.VoidType(),
-                                   [void_ptr, void_ptr])
-        fn = mod.get_or_insert_function(fnty, name="NRT_MemInfo_varsize_free")
-        return builder.call(fn, (meminfo, ptr))
-
-    def _call_nrt_varsize_alloc(self, builder, meminfo, size, funcname):
-        self._require_nrt()
-
-        mod = builder.module
-        fnty = llvmir.FunctionType(void_ptr,
-                                   [void_ptr, self.get_value_type(types.intp)])
-        fn = mod.get_or_insert_function(fnty, name=funcname)
-        fn.return_value.add_attribute("noalias")
-        return builder.call(fn, [meminfo, size])
-
-    def nrt_meminfo_data(self, builder, meminfo):
-        """
-        Given a MemInfo pointer, return a pointer to the allocated data
-        managed by it.  This works for MemInfos allocated with all the
-        above methods.
-        """
-        self._require_nrt()
-
-        from numba.runtime.atomicops import meminfo_data_ty
-
-        mod = builder.module
-        fn = mod.get_or_insert_function(meminfo_data_ty,
-                                        name="NRT_MemInfo_data_fast")
-        return builder.call(fn, [meminfo])
-
-    def _call_nrt_incref_decref(self, builder, root_type, typ, value,
-                                funcname, getters=()):
-        self._require_nrt()
-
-        from numba.runtime.atomicops import incref_decref_ty
-
-        data_model = self.data_model_manager[typ]
-
-        members = data_model.traverse(builder)
-        for mtyp, getter in members:
-            self._call_nrt_incref_decref(builder, root_type, mtyp, value,
-                                         funcname, getters + (getter,))
-
-        if data_model.has_nrt_meminfo():
-            # Call the chain of getters to compute the member value
-            for getter in getters:
-                value = getter(value)
-            try:
-                meminfo = data_model.get_nrt_meminfo(builder, value)
-            except NotImplementedError as e:
-                raise NotImplementedError("%s: %s" % (root_type, str(e)))
-            assert meminfo is not None  # since has_nrt_meminfo()
-            mod = builder.module
-            fn = mod.get_or_insert_function(incref_decref_ty, name=funcname)
-            # XXX "nonnull" causes a crash in test_dyn_array: can this
-            # function be called with a NULL pointer?
-            fn.args[0].add_attribute("noalias")
-            fn.args[0].add_attribute("nocapture")
-            builder.call(fn, [meminfo])
-
-    def nrt_incref(self, builder, typ, value):
-        """
-        Recursively incref the given *value* and its members.
-        """
-        self._call_nrt_incref_decref(builder, typ, typ, value, "NRT_incref")
-
-    def nrt_decref(self, builder, typ, value):
-        """
-        Recursively decref the given *value* and its members.
-        """
-        self._call_nrt_incref_decref(builder, typ, typ, value, "NRT_decref")
 
 
 class _wrap_impl(object):
