@@ -2,7 +2,6 @@ from __future__ import print_function, division, absolute_import
 
 import functools
 import locale
-import sys
 import weakref
 
 import llvmlite.llvmpy.core as lc
@@ -391,8 +390,11 @@ class JITCodeLibrary(CodeLibrary):
 
 
 class BaseCPUCodegen(object):
+    _llvm_initialized = False
 
     def __init__(self, module_name):
+        initialize_llvm()
+
         self._libraries = set()
         self._data_layout = None
         self._llvm_module = ll.parse_assembly(
@@ -420,7 +422,7 @@ class BaseCPUCodegen(object):
                                       self._library_class._object_getbuffer_hook)
 
     def _create_empty_module(self, name):
-        ir_module = lc.Module.new(name)
+        ir_module = lc.Module(name)
         ir_module.triple = ll.get_process_triple()
         if self._data_layout:
             ir_module.data_layout = self._data_layout
@@ -527,13 +529,23 @@ class AOTCPUCodegen(BaseCPUCodegen):
 
     _library_class = AOTCodeLibrary
 
+    def __init__(self, module_name, cpu_name=None):
+        # By default, use generic cpu model for the arch
+        self._cpu_name = cpu_name or ''
+        BaseCPUCodegen.__init__(self, module_name)
+
     def _customize_tm_options(self, options):
-        options['cpu'] = ''  # use generic cpu model for the arch
+        cpu_name = self._cpu_name
+        if cpu_name == 'host':
+            cpu_name = ll.get_host_cpu_name()
+        options['cpu'] = cpu_name
         options['reloc'] = 'pic'
         options['codemodel'] = 'default'
         options['features'] = self._tm_features
 
     def _customize_tm_features(self):
+        # ISA features are selected according to the requested CPU model
+        # in _customize_tm_options()
         return ''
 
     def _add_module(self, module):
@@ -555,7 +567,8 @@ class JITCPUCodegen(BaseCPUCodegen):
         options['reloc'] = 'default'
         options['codemodel'] = 'jitdefault'
 
-        # Set feature attributes
+        # Set feature attributes (such as ISA extensions)
+        # This overrides default feature selection by CPU model above
         options['features'] = self._tm_features
 
         # Enable JIT debug
@@ -565,11 +578,8 @@ class JITCPUCodegen(BaseCPUCodegen):
         # For JIT target, we will use LLVM to get the feature map
         features = ll.get_host_cpu_features()
 
-        # There are various performance issues with AVX and LLVM 3.5
-        # (list at http://llvm.org/bugs/buglist.cgi?quicksearch=avx).
-        # For now we'd rather disable it, since it can pessimize the code.
         if not config.ENABLE_AVX:
-            # Disable all feature with name starting with 'avx'
+            # Disable all features with name starting with 'avx'
             for k in features:
                 if k.startswith('avx'):
                     features[k] = False
@@ -581,3 +591,14 @@ class JITCPUCodegen(BaseCPUCodegen):
         self._engine.add_module(module)
         # Early bind the engine method to avoid keeping a reference to self.
         return functools.partial(self._engine.remove_module, module)
+
+
+_llvm_initialized = False
+
+def initialize_llvm():
+    global _llvm_initialized
+    if not _llvm_initialized:
+        ll.initialize()
+        ll.initialize_native_target()
+        ll.initialize_native_asmprinter()
+        _llvm_initialized = True
