@@ -8,6 +8,8 @@ from numba.errors import TypingError
 from numba import types
 
 
+a0 = np.array(42)
+
 s1 = np.int32(64)
 
 a1 = np.arange(12)
@@ -20,6 +22,10 @@ a4 = np.arange(32, dtype=np.int8).view(dt)
 a5 = a4[::-2]
 
 
+def getitem0(i):
+    return a0[()]
+
+
 def getitem1(i):
     return a1[i]
 
@@ -56,6 +62,9 @@ class TestConstantArray(unittest.TestCase):
         for i in [0, 1, 2]:
             np.testing.assert_array_equal(pyfunc(i), cfunc(i))
 
+    def test_array_const_0d(self):
+        self.check_array_const(getitem0)
+
     def test_array_const_1d_contig(self):
         self.check_array_const(getitem1)
 
