@@ -4,6 +4,7 @@ Tests for numba.types.
 
 from __future__ import print_function, absolute_import
 
+from collections import namedtuple
 import gc
 try:
     import cPickle as pickle
@@ -13,18 +14,22 @@ import weakref
 
 import numpy as np
 
+from numba import unittest_support as unittest
 from numba.utils import IS_PY3
-from numba import types, typing
+from numba import sigutils, types, typing
 from numba.types.abstract import _typecache
 from numba import jit, numpy_support
-from numba import unittest_support as unittest
 from .support import TestCase, tag
+from .enum_usecases import *
 
 
+Point = namedtuple('Point', ('x', 'y'))
+
+Rect = namedtuple('Rect', ('width', 'height'))
+
 def gen(x):
     yield x + 1
 
-
 class Dummy(object):
     pass
 
@@ -403,6 +408,20 @@ class TestPickling(TestCase):
         ty2 = types.Tuple((types.int32, ty1))
         self.check_pickling(ty2)
 
+    def test_namedtuples(self):
+        ty1 = types.NamedUniTuple(types.intp, 2, Point)
+        self.check_pickling(ty1)
+        ty2 = types.NamedTuple((types.intp, types.float64), Point)
+        self.check_pickling(ty2)
+
+    def test_enums(self):
+        ty1 = types.EnumMember(Color, types.int32)
+        self.check_pickling(ty1)
+        ty2 = types.EnumMember(Shake, types.int64)
+        self.check_pickling(ty2)
+        ty3 = types.IntEnumMember(Shape, types.int64)
+        self.check_pickling(ty3)
+
     def test_lists(self):
         ty = types.List(types.int32)
         self.check_pickling(ty)
@@ -425,5 +444,35 @@ class TestPickling(TestCase):
             self.check_pickling(ty)
 
 
+class TestSignatures(TestCase):
+
+    def test_normalize_signature(self):
+        f = sigutils.normalize_signature
+
+        def check(sig, args, return_type):
+            self.assertEqual(f(sig), (args, return_type))
+
+        def check_error(sig, msg):
+            with self.assertRaises(TypeError) as raises:
+                f(sig)
+            self.assertIn(msg, str(raises.exception))
+
+        f32 = types.float32
+        c64 = types.complex64
+        i16 = types.int16
+        a = types.Array(f32, 1, "C")
+
+        check((c64,), (c64,), None)
+        check((f32, i16), (f32, i16), None)
+        check(a(i16), (i16,), a)
+        check("int16(complex64)", (c64,), i16)
+        check("(complex64, int16)", (c64, i16), None)
+        check(typing.signature(i16, c64), (c64,), i16)
+
+        check_error((types.Integer,), "invalid signature")
+        check_error((None,), "invalid signature")
+        check_error([], "invalid signature")
+
+
 if __name__ == '__main__':
     unittest.main()
