@@ -1,13 +1,17 @@
 from __future__ import print_function
 
+import math
+import re
+import textwrap
+
+import numpy as np
+
 import numba.unittest_support as unittest
 from numba.compiler import compile_isolated
-from numba import types
+from numba import jit, types
 from numba.errors import TypingError
 from .support import TestCase
 
-import math
-
 
 def what():
     pass
@@ -37,6 +41,14 @@ def imprecise_list():
 def unknown_module():
     return numpyz.int32(0)
 
+def nop(x, y, z):
+    pass
+
+
+class Foo(object):
+    def __repr__(self):
+        return "<Foo instance>"
+
 
 class TestTypingError(unittest.TestCase):
 
@@ -68,12 +80,14 @@ class TestTypingError(unittest.TestCase):
         type inference because TimeDeltaMixOp always assumed at least one of
         its operands was an NPTimeDelta in its generic() method.
         '''
-        try:
+        with self.assertRaises(TypingError) as raises:
             compile_isolated(issue_868, (types.Array(types.int32, 1, 'C'),))
-        except TypingError as e:
-            self.assertTrue(e.msg.startswith('Invalid usage of * '))
-        else:
-            self.fail('Should raise error')
+
+        expected = (
+            "Invalid usage of * with parameters (({0} x 1), {0})"
+            .format(str(types.intp)))
+        self.assertIn(expected, str(raises.exception))
+        self.assertIn("[1] During: typing of", str(raises.exception))
 
     def test_return_type_unification(self):
         with self.assertRaises(TypingError) as raises:
@@ -90,6 +104,12 @@ class TestTypingError(unittest.TestCase):
         # This is sensitive to the formatting of the error message.
         self.assertIn(" * (float64, float64) -> float64", errmsg)
 
+        # Check contextual msg
+        last_two = errmsg.splitlines()[-2:]
+        self.assertTrue(re.search(r'\[1\] During: resolving callee type: Function.*hypot', last_two[0]))
+        self.assertTrue(re.search(r'\[2\] During: typing of call .*test_typingerror.py', last_two[1]))
+
+
     def test_imprecise_list(self):
         """
         Type inference should catch that a list type's remain imprecise,
@@ -103,5 +123,36 @@ class TestTypingError(unittest.TestCase):
                       errmsg)
 
 
+class TestArgumentTypingError(unittest.TestCase):
+    """
+    Test diagnostics of typing errors caused by argument inference failure.
+    """
+
+    def test_unsupported_array_dtype(self):
+        # See issue #1943
+        cfunc = jit(nopython=True)(nop)
+        a = np.ones(3)
+        a = a.astype(a.dtype.newbyteorder())
+        with self.assertRaises(TypingError) as raises:
+            cfunc(1, a, a)
+        expected = textwrap.dedent("""\
+            This error may have been caused by the following argument(s):
+            - argument 1: Unsupported array dtype: {0}
+            - argument 2: Unsupported array dtype: {0}"""
+            ).format(a.dtype)
+        self.assertIn(expected, str(raises.exception))
+
+    def test_unsupported_type(self):
+        cfunc = jit(nopython=True)(nop)
+        foo = Foo()
+        with self.assertRaises(TypingError) as raises:
+            cfunc(1, foo, 1)
+        expected = textwrap.dedent("""\
+            This error may have been caused by the following argument(s):
+            - argument 1: cannot determine Numba type of value <Foo instance>"""
+            )
+        self.assertIn(expected, str(raises.exception))
+
+
 if __name__ == '__main__':
     unittest.main()
