@@ -1396,7 +1396,11 @@ class _LoopTypesTester(TestCase):
     """
     _skip_types = 'OegG'
 
+    # Allowed deviation between Numpy and Numba results
     _ulps = {('arccos', 'F'): 2,
+             ('arcsin', 'D'): 4,
+             ('arcsin', 'F'): 4,
+             ('log10', 'D'): 5,
              ('tanh', 'F'): 2,
              }
 
