@@ -14,12 +14,13 @@ Constraints push types forward following the dataflow.
 
 from __future__ import print_function, division, absolute_import
 
-from pprint import pprint
+import contextlib
 import itertools
+from pprint import pprint
 import traceback
 
-from numba import ir, types, utils, config, six
-from .errors import TypingError, UntypedAttributeError
+from numba import ir, types, utils, config, six, typing
+from .errors import TypingError, UntypedAttributeError, new_error_context
 
 
 class TypeVar(object):
@@ -28,28 +29,38 @@ class TypeVar(object):
         self.var = var
         self.type = None
         self.locked = False
+        # Stores source location of first definition
+        self.define_loc = None
 
-    def add_type(self, tp):
+    def add_type(self, tp, loc):
         assert isinstance(tp, types.Type), type(tp)
 
         if self.locked:
             if tp != self.type:
                 if self.context.can_convert(tp, self.type) is None:
-                    raise TypingError("No conversion from %s to %s for "
-                                      "'%s'" % (tp, self.type, self.var))
+                    msg = ("No conversion from %s to %s for '%s', "
+                           "defined at %s")
+                    raise TypingError(msg % (tp, self.type, self.var,
+                                             self.define_loc),
+                                      loc=loc)
         else:
             if self.type is not None:
                 unified = self.context.unify_pairs(self.type, tp)
                 if unified is None:
-                    raise TypingError("cannot unify %s and %s for '%s'"
-                                      % (self.type, tp, self.var))
+                    msg = "cannot unify %s and %s for '%s', defined at %s"
+                    raise TypingError(msg % (self.type, tp, self.var,
+                                             self.define_loc),
+                                      loc=loc)
             else:
+                # First time definition
                 unified = tp
+                self.define_loc = loc
+
             self.type = unified
 
         return self.type
 
-    def lock(self, tp):
+    def lock(self, tp, loc):
         assert isinstance(tp, types.Type), type(tp)
         assert not self.locked
 
@@ -58,14 +69,16 @@ class TypeVar(object):
         if (self.type is not None and
             self.context.can_convert(self.type, tp) is None):
             raise TypingError("No conversion from %s to %s for "
-                                "'%s'" % (tp, self.type, self.var))
+                              "'%s'" % (tp, self.type, self.var))
 
         self.type = tp
         self.locked = True
+        if self.define_loc is None:
+            self.define_loc = loc
 
-    def union(self, other):
+    def union(self, other, loc):
         if other.type is not None:
-            self.add_type(other.type)
+            self.add_type(other.type, loc=loc)
 
         return self.type
 
@@ -135,14 +148,15 @@ class Propagate(object):
         self.loc = loc
 
     def __call__(self, typeinfer):
-        typevars = typeinfer.typevars
-        typeinfer.copy_type(self.src, self.dst)
-        # If `dst` is refined, notify us
-        typeinfer.refine_map[self.dst] = self
+        with new_error_context("typing of assignment at {0}", self.loc):
+            typeinfer.copy_type(self.src, self.dst, loc=self.loc)
+            # If `dst` is refined, notify us
+            typeinfer.refine_map[self.dst] = self
 
     def refine(self, typeinfer, target_type):
         # Do not back-propagate to locked variables (e.g. constants)
-        typeinfer.add_type(self.src, target_type, unless_locked=True)
+        typeinfer.add_type(self.src, target_type, unless_locked=True,
+                           loc=self.loc)
 
 
 class ArgConstraint(object):
@@ -153,14 +167,15 @@ class ArgConstraint(object):
         self.loc = loc
 
     def __call__(self, typeinfer):
-        typevars = typeinfer.typevars
-        src = typevars[self.src]
-        if not src.defined:
-            return
-        ty = src.getone()
-        if isinstance(ty, types.Omitted):
-            ty = typeinfer.context.resolve_value_type(ty.value)
-        typeinfer.add_type(self.dst, ty)
+        with new_error_context("typing of argument at {0}", self.loc):
+            typevars = typeinfer.typevars
+            src = typevars[self.src]
+            if not src.defined:
+                return
+            ty = src.getone()
+            if isinstance(ty, types.Omitted):
+                ty = typeinfer.context.resolve_value_type(ty.value)
+            typeinfer.add_type(self.dst, ty, loc=self.loc)
 
 
 class BuildTupleConstraint(object):
@@ -170,16 +185,17 @@ class BuildTupleConstraint(object):
         self.loc = loc
 
     def __call__(self, typeinfer):
-        typevars = typeinfer.typevars
-        tsets = [typevars[i.name].get() for i in self.items]
-        oset = typevars[self.target]
-        for vals in itertools.product(*tsets):
-            if vals and all(vals[0] == v for v in vals):
-                tup = types.UniTuple(dtype=vals[0], count=len(vals))
-            else:
-                # empty tuples fall here as well
-                tup = types.Tuple(vals)
-            typeinfer.add_type(self.target, tup)
+        with new_error_context("typing of tuple at {0}", self.loc):
+            typevars = typeinfer.typevars
+            tsets = [typevars[i.name].get() for i in self.items]
+            oset = typevars[self.target]
+            for vals in itertools.product(*tsets):
+                if vals and all(vals[0] == v for v in vals):
+                    tup = types.UniTuple(dtype=vals[0], count=len(vals))
+                else:
+                    # empty tuples fall here as well
+                    tup = types.Tuple(vals)
+                typeinfer.add_type(self.target, tup, loc=self.loc)
 
 
 class _BuildContainerConstraint(object):
@@ -190,18 +206,21 @@ class _BuildContainerConstraint(object):
         self.loc = loc
 
     def __call__(self, typeinfer):
-        typevars = typeinfer.typevars
-        oset = typevars[self.target]
-        tsets = [typevars[i.name].get() for i in self.items]
-        if not tsets:
-            typeinfer.add_type(self.target,
-                               self.container_type(types.undefined))
-        else:
-            for typs in itertools.product(*tsets):
-                unified = typeinfer.context.unify_types(*typs)
-                if unified is not None:
-                    typeinfer.add_type(self.target,
-                                       self.container_type(unified))
+        with new_error_context("typing of list at {0}", self.loc):
+            typevars = typeinfer.typevars
+            oset = typevars[self.target]
+            tsets = [typevars[i.name].get() for i in self.items]
+            if not tsets:
+                typeinfer.add_type(self.target,
+                                   self.container_type(types.undefined),
+                                   loc=self.loc)
+            else:
+                for typs in itertools.product(*tsets):
+                    unified = typeinfer.context.unify_types(*typs)
+                    if unified is not None:
+                        typeinfer.add_type(self.target,
+                                           self.container_type(unified),
+                                           loc=self.loc)
 
 
 class BuildListConstraint(_BuildContainerConstraint):
@@ -212,6 +231,7 @@ class BuildSetConstraint(_BuildContainerConstraint):
     container_type = types.Set
 
 
+
 class ExhaustIterConstraint(object):
     def __init__(self, target, count, iterator, loc):
         self.target = target
@@ -220,20 +240,21 @@ class ExhaustIterConstraint(object):
         self.loc = loc
 
     def __call__(self, typeinfer):
-        typevars = typeinfer.typevars
-        oset = typevars[self.target]
-        for tp in typevars[self.iterator.name].get():
-            if isinstance(tp, types.BaseTuple):
-                if len(tp) == self.count:
-                    typeinfer.add_type(self.target, tp)
-                else:
-                    raise ValueError("wrong tuple length for %r: "
-                                     "expected %d, got %d"
-                                     % (self.iterator.name, self.count, len(tp)))
-            elif isinstance(tp, types.IterableType):
-                tup = types.UniTuple(dtype=tp.iterator_type.yield_type,
-                                     count=self.count)
-                typeinfer.add_type(self.target, tup)
+        with new_error_context("typing of exhaust iter at {0}", self.loc):
+            typevars = typeinfer.typevars
+            oset = typevars[self.target]
+            for tp in typevars[self.iterator.name].get():
+                if isinstance(tp, types.BaseTuple):
+                    if len(tp) == self.count:
+                        typeinfer.add_type(self.target, tp, loc=self.loc)
+                    else:
+                        raise ValueError("wrong tuple length for %r: "
+                                         "expected %d, got %d"
+                                         % (self.iterator.name, self.count, len(tp)))
+                elif isinstance(tp, types.IterableType):
+                    tup = types.UniTuple(dtype=tp.iterator_type.yield_type,
+                                         count=self.count)
+                    typeinfer.add_type(self.target, tup, loc=self.loc)
 
 
 class PairFirstConstraint(object):
@@ -243,13 +264,14 @@ class PairFirstConstraint(object):
         self.loc = loc
 
     def __call__(self, typeinfer):
-        typevars = typeinfer.typevars
-        oset = typevars[self.target]
-        for tp in typevars[self.pair.name].get():
-            if not isinstance(tp, types.Pair):
-                # XXX is this an error?
-                continue
-            typeinfer.add_type(self.target, tp.first_type)
+        with new_error_context("typing of pair-first at {0}", self.loc):
+            typevars = typeinfer.typevars
+            oset = typevars[self.target]
+            for tp in typevars[self.pair.name].get():
+                if not isinstance(tp, types.Pair):
+                    # XXX is this an error?
+                    continue
+                typeinfer.add_type(self.target, tp.first_type, loc=self.loc)
 
 
 class PairSecondConstraint(object):
@@ -259,13 +281,14 @@ class PairSecondConstraint(object):
         self.loc = loc
 
     def __call__(self, typeinfer):
-        typevars = typeinfer.typevars
-        oset = typevars[self.target]
-        for tp in typevars[self.pair.name].get():
-            if not isinstance(tp, types.Pair):
-                # XXX is this an error?
-                continue
-            typeinfer.add_type(self.target, tp.second_type)
+        with new_error_context("typing of pair-second at {0}", self.loc):
+            typevars = typeinfer.typevars
+            oset = typevars[self.target]
+            for tp in typevars[self.pair.name].get():
+                if not isinstance(tp, types.Pair):
+                    # XXX is this an error?
+                    continue
+                typeinfer.add_type(self.target, tp.second_type, loc=self.loc)
 
 
 class StaticGetItemConstraint(object):
@@ -282,21 +305,51 @@ class StaticGetItemConstraint(object):
         self.loc = loc
 
     def __call__(self, typeinfer):
-        typevars = typeinfer.typevars
-        oset = typevars[self.target]
-        for ty in typevars[self.value.name].get():
-            itemty = typeinfer.context.resolve_static_getitem(value=ty,
-                                                              index=self.index)
-            if itemty is not None:
-                typeinfer.add_type(self.target, itemty)
-            elif self.fallback is not None:
-                self.fallback(typeinfer)
+        with new_error_context("typing of static-get-item at {0}", self.loc):
+            typevars = typeinfer.typevars
+            oset = typevars[self.target]
+            for ty in typevars[self.value.name].get():
+                itemty = typeinfer.context.resolve_static_getitem(value=ty,
+                                                                  index=self.index)
+                if itemty is not None:
+                    typeinfer.add_type(self.target, itemty, loc=self.loc)
+                elif self.fallback is not None:
+                    self.fallback(typeinfer)
 
     def get_call_signature(self):
         # The signature is only needed for the fallback case in lowering
         return self.fallback and self.fallback.get_call_signature()
 
 
+def fold_arg_vars(typevars, args, vararg, kws):
+    """
+    Fold and resolve the argument variables of a function call.
+    """
+    # Fetch all argument types, bail if any is unknown
+    n_pos_args = len(args)
+    kwds = [kw for (kw, var) in kws]
+    argtypes = [typevars[a.name] for a in args]
+    argtypes += [typevars[var.name] for (kw, var) in kws]
+    if vararg is not None:
+        argtypes.append(typevars[vararg.name])
+
+    if not all(a.defined for a in argtypes):
+        return
+
+    args = tuple(a.getone() for a in argtypes)
+    pos_args = args[:n_pos_args]
+    if vararg is not None:
+        if not isinstance(args[-1], types.BaseTuple):
+            # Unsuitable for *args
+            # (Python is more lenient and accepts all iterables)
+            raise TypeError("*args in function call should be a tuple, got %s"
+                            % (args[-1],))
+        pos_args += args[-1].types
+        args = args[:-1]
+    kw_args = dict(zip(kwds, args[n_pos_args:]))
+    return pos_args, kw_args
+
+
 class CallConstraint(object):
     """Constraint for calling functions.
     Perform case analysis foreach combinations of argument types.
@@ -312,43 +365,35 @@ class CallConstraint(object):
         self.loc = loc
 
     def __call__(self, typeinfer):
-        typevars = typeinfer.typevars
-        fnty = typevars[self.func].getone()
-        self.resolve(typeinfer, typevars, fnty)
+        with new_error_context("typing of call at {0}", self.loc):
+            typevars = typeinfer.typevars
+            fnty = typevars[self.func].getone()
+            with new_error_context("resolving callee type: {0}", fnty):
+                self.resolve(typeinfer, typevars, fnty)
 
     def resolve(self, typeinfer, typevars, fnty):
         assert fnty
         context = typeinfer.context
 
-        n_pos_args = len(self.args)
-        kwds = [kw for (kw, var) in self.kws]
-        argtypes = [typevars[a.name] for a in self.args]
-        argtypes += [typevars[var.name] for (kw, var) in self.kws]
-        if self.vararg is not None:
-            argtypes.append(typevars[self.vararg.name])
-
-        if not all(a.defined for a in argtypes):
+        r = fold_arg_vars(typevars, self.args, self.vararg, self.kws)
+        if r is None:
             # Cannot resolve call type until all argument types are known
             return
+        pos_args, kw_args = r
 
-        args = tuple(a.getone() for a in argtypes)
-        pos_args = args[:n_pos_args]
-        if self.vararg is not None:
-            if not isinstance(args[-1], types.BaseTuple):
-                # Unsuitable for *args
-                # (Python is more lenient and accepts all iterables)
-                return
-            pos_args += args[-1].types
-            args = args[:-1]
-        kw_args = dict(zip(kwds, args[n_pos_args:]))
-        sig = context.resolve_function_type(fnty, pos_args, kw_args)
+        # Resolve call type
+        sig = typeinfer.resolve_call(fnty, pos_args, kw_args)
         if sig is None:
-            desc = context.explain_function_type(fnty)
+            # Arguments are invalid => explain why
             headtemp = "Invalid usage of {0} with parameters ({1})"
+            args = [str(a) for a in pos_args]
+            args += ["%s=%s" % (k, v) for k, v in sorted(kw_args.items())]
             head = headtemp.format(fnty, ', '.join(map(str, args)))
+            desc = context.explain_function_type(fnty)
             msg = '\n'.join([head, desc])
             raise TypingError(msg, loc=self.loc)
-        typeinfer.add_type(self.target, sig.return_type)
+
+        typeinfer.add_type(self.target, sig.return_type, loc=self.loc)
 
         # If the function is a bound function and its receiver type
         # was refined, propagate it.
@@ -381,7 +426,8 @@ class CallConstraint(object):
 
 class IntrinsicCallConstraint(CallConstraint):
     def __call__(self, typeinfer):
-        self.resolve(typeinfer, typeinfer.typevars, fnty=self.func)
+        with new_error_context("typing of intrinsic-call at {0}", self.loc):
+            self.resolve(typeinfer, typeinfer.typevars, fnty=self.func)
 
 
 class GetAttrConstraint(object):
@@ -393,20 +439,21 @@ class GetAttrConstraint(object):
         self.inst = inst
 
     def __call__(self, typeinfer):
-        typevars = typeinfer.typevars
-        valtys = typevars[self.value.name].get()
-        for ty in valtys:
-            attrty = typeinfer.context.resolve_getattr(ty, self.attr)
-            if attrty is None:
-                raise UntypedAttributeError(ty, self.attr, loc=self.inst.loc)
-            else:
-                typeinfer.add_type(self.target, attrty)
-        typeinfer.refine_map[self.target] = self
+        with new_error_context("typing of get attribute at {0}", self.loc):
+            typevars = typeinfer.typevars
+            valtys = typevars[self.value.name].get()
+            for ty in valtys:
+                attrty = typeinfer.context.resolve_getattr(ty, self.attr)
+                if attrty is None:
+                    raise UntypedAttributeError(ty, self.attr, loc=self.inst.loc)
+                else:
+                    typeinfer.add_type(self.target, attrty, loc=self.loc)
+            typeinfer.refine_map[self.target] = self
 
     def refine(self, typeinfer, target_type):
         if isinstance(target_type, types.BoundFunction):
             recvr = target_type.this
-            typeinfer.add_type(self.value.name, recvr)
+            typeinfer.add_type(self.value.name, recvr, loc=self.loc)
             source_constraint = typeinfer.refine_map.get(self.value.name)
             if source_constraint is not None:
                 source_constraint.refine(typeinfer, recvr)
@@ -424,19 +471,20 @@ class SetItemConstraint(object):
         self.loc = loc
 
     def __call__(self, typeinfer):
-        typevars = typeinfer.typevars
-        if not all(typevars[var.name].defined
-                   for var in (self.target, self.index, self.value)):
-            return
-        targetty = typevars[self.target.name].getone()
-        idxty = typevars[self.index.name].getone()
-        valty = typevars[self.value.name].getone()
+        with new_error_context("typing of setitem at {0}", self.loc):
+            typevars = typeinfer.typevars
+            if not all(typevars[var.name].defined
+                       for var in (self.target, self.index, self.value)):
+                return
+            targetty = typevars[self.target.name].getone()
+            idxty = typevars[self.index.name].getone()
+            valty = typevars[self.value.name].getone()
 
-        sig = typeinfer.context.resolve_setitem(targetty, idxty, valty)
-        if sig is None:
-            raise TypingError("Cannot resolve setitem: %s[%s] = %s" %
-                              (targetty, idxty, valty), loc=self.loc)
-        self.signature = sig
+            sig = typeinfer.context.resolve_setitem(targetty, idxty, valty)
+            if sig is None:
+                raise TypingError("Cannot resolve setitem: %s[%s] = %s" %
+                                  (targetty, idxty, valty), loc=self.loc)
+            self.signature = sig
 
     def get_call_signature(self):
         return self.signature
@@ -451,21 +499,22 @@ class StaticSetItemConstraint(object):
         self.loc = loc
 
     def __call__(self, typeinfer):
-        typevars = typeinfer.typevars
-        if not all(typevars[var.name].defined
-                   for var in (self.target, self.index_var, self.value)):
-            return
-        targetty = typevars[self.target.name].getone()
-        idxty = typevars[self.index_var.name].getone()
-        valty = typevars[self.value.name].getone()
+        with new_error_context("typing of staticsetitem at {0}", self.loc):
+            typevars = typeinfer.typevars
+            if not all(typevars[var.name].defined
+                       for var in (self.target, self.index_var, self.value)):
+                return
+            targetty = typevars[self.target.name].getone()
+            idxty = typevars[self.index_var.name].getone()
+            valty = typevars[self.value.name].getone()
 
-        sig = typeinfer.context.resolve_static_setitem(targetty, self.index, valty)
-        if sig is None:
-            sig = typeinfer.context.resolve_setitem(targetty, idxty, valty)
-        if sig is None:
-            raise TypingError("Cannot resolve setitem: %s[%r] = %s" %
-                              (targetty, self.index, valty), loc=self.loc)
-        self.signature = sig
+            sig = typeinfer.context.resolve_static_setitem(targetty, self.index, valty)
+            if sig is None:
+                sig = typeinfer.context.resolve_setitem(targetty, idxty, valty)
+            if sig is None:
+                raise TypingError("Cannot resolve setitem: %s[%r] = %s" %
+                                  (targetty, self.index, valty), loc=self.loc)
+            self.signature = sig
 
     def get_call_signature(self):
         return self.signature
@@ -478,18 +527,19 @@ class DelItemConstraint(object):
         self.loc = loc
 
     def __call__(self, typeinfer):
-        typevars = typeinfer.typevars
-        if not all(typevars[var.name].defined
-                   for var in (self.target, self.index)):
-            return
-        targetty = typevars[self.target.name].getone()
-        idxty = typevars[self.index.name].getone()
+        with new_error_context("typing of delitem at {0}", self.loc):
+            typevars = typeinfer.typevars
+            if not all(typevars[var.name].defined
+                       for var in (self.target, self.index)):
+                return
+            targetty = typevars[self.target.name].getone()
+            idxty = typevars[self.index.name].getone()
 
-        sig = typeinfer.context.resolve_delitem(targetty, idxty)
-        if sig is None:
-            raise TypingError("Cannot resolve delitem: %s[%s]" %
-                              (targetty, idxty), loc=self.loc)
-        self.signature = sig
+            sig = typeinfer.context.resolve_delitem(targetty, idxty)
+            if sig is None:
+                raise TypingError("Cannot resolve delitem: %s[%s]" %
+                                  (targetty, idxty), loc=self.loc)
+            self.signature = sig
 
     def get_call_signature(self):
         return self.signature
@@ -503,17 +553,44 @@ class SetAttrConstraint(object):
         self.loc = loc
 
     def __call__(self, typeinfer):
-        typevars = typeinfer.typevars
-        if not all(typevars[var.name].defined
-                   for var in (self.target, self.value)):
-            return
-        targetty = typevars[self.target.name].getone()
-        valty = typevars[self.value.name].getone()
+        with new_error_context("typing of set attribute {0!r} at {1}",
+                               self.attr, self.loc):
+            typevars = typeinfer.typevars
+            if not all(typevars[var.name].defined
+                       for var in (self.target, self.value)):
+                return
+            targetty = typevars[self.target.name].getone()
+            valty = typevars[self.value.name].getone()
+            sig = typeinfer.context.resolve_setattr(targetty, self.attr,
+                                                    valty)
+            if sig is None:
+                raise TypingError("Cannot resolve setattr: (%s).%s = %s" %
+                                  (targetty, self.attr, valty),
+                                  loc=self.loc)
+            self.signature = sig
 
-        sig = typeinfer.context.resolve_setattr(targetty, self.attr, valty)
-        if sig is None:
-            raise TypingError("Cannot resolve setattr: (%s).%s = %s" %
-                              (targetty, self.attr, valty), loc=self.loc)
+    def get_call_signature(self):
+        return self.signature
+
+
+class PrintConstraint(object):
+    def __init__(self, args, vararg, loc):
+        self.args = args
+        self.vararg = vararg
+        self.loc = loc
+
+    def __call__(self, typeinfer):
+        typevars = typeinfer.typevars
+
+        r = fold_arg_vars(typevars, self.args, self.vararg, {})
+        if r is None:
+            # Cannot resolve call type until all argument types are known
+            return
+        pos_args, kw_args = r
+
+        fnty = typeinfer.context.resolve_value_type(print)
+        assert fnty is not None
+        sig = typeinfer.resolve_call(fnty, pos_args, kw_args)
         self.signature = sig
 
     def get_call_signature(self):
@@ -537,6 +614,27 @@ class TypeVarMap(dict):
             super(TypeVarMap, self).__setitem__(name, value)
 
 
+# A temporary mapping of {function name: dispatcher object}
+_temporary_dispatcher_map = {}
+
+@contextlib.contextmanager
+def register_dispatcher(disp):
+    """
+    Register a Dispatcher for inference while it is not yet stored
+    as global or closure variable (e.g. during execution of the @jit()
+    call).  This allows resolution of recursive calls with eager
+    compilation.
+    """
+    assert callable(disp)
+    assert callable(disp.py_func)
+    name = disp.py_func.__name__
+    _temporary_dispatcher_map[name] = disp
+    try:
+        yield
+    finally:
+        del _temporary_dispatcher_map[name]
+
+
 class TypeInferer(object):
     """
     Operates on block that shares the same ir.Scope.
@@ -554,7 +652,7 @@ class TypeInferer(object):
 
         # { index: mangled name }
         self.arg_names = {}
-        self.return_type = None
+        #self.return_type = None
         # Set of assumed immutable globals
         self.assumed_immutables = set()
         # Track all calls and associated constraints
@@ -573,6 +671,14 @@ class TypeInferer(object):
         # Disambiguise argument name
         return "arg.%s" % (name,)
 
+    def _get_return_vars(self):
+        rets = []
+        for blk in utils.itervalues(self.blocks):
+            inst = blk.terminator
+            if isinstance(inst, ir.Return):
+                rets.append(inst.value)
+        return rets
+
     def seed_argument(self, name, index, typ):
         name = self._mangle_arg_name(name)
         self.seed_type(name, typ)
@@ -581,15 +687,13 @@ class TypeInferer(object):
     def seed_type(self, name, typ):
         """All arguments should be seeded.
         """
-        self.lock_type(name, typ)
+        self.lock_type(name, typ, loc=None)
 
     def seed_return(self, typ):
         """Seeding of return value is optional.
         """
-        for blk in utils.itervalues(self.blocks):
-            inst = blk.terminator
-            if isinstance(inst, ir.Return):
-                self.lock_type(inst.value.name, typ)
+        for var in self._get_return_vars():
+            self.lock_type(var.name, typ, loc=None)
 
     def build_constraint(self):
         for blk in utils.itervalues(self.blocks):
@@ -612,25 +716,25 @@ class TypeInferer(object):
         if errors:
             raise errors[0]
 
-    def add_type(self, var, tp, unless_locked=False):
+    def add_type(self, var, tp, loc, unless_locked=False):
         assert isinstance(var, str), type(var)
         tv = self.typevars[var]
         if unless_locked and tv.locked:
             return
         oldty = tv.type
-        unified = tv.add_type(tp)
+        unified = tv.add_type(tp, loc=loc)
         if unified != oldty:
             self.propagate_refined_type(var, unified)
 
     def add_calltype(self, inst, signature):
         self.calltypes[inst] = signature
 
-    def copy_type(self, src_var, dest_var):
-        unified = self.typevars[dest_var].union(self.typevars[src_var])
+    def copy_type(self, src_var, dest_var, loc):
+        unified = self.typevars[dest_var].union(self.typevars[src_var], loc=loc)
 
-    def lock_type(self, var, tp):
+    def lock_type(self, var, tp, loc):
         tv = self.typevars[var]
-        tv.lock(tp)
+        tv.lock(tp, loc=loc)
 
     def propagate_refined_type(self, updated_var, updated_type):
         source_constraint = self.refine_map.get(updated_var)
@@ -697,13 +801,7 @@ class TypeInferer(object):
             calltypes[call] = constraint.get_call_signature()
         return calltypes
 
-    def get_return_type(self, typemap):
-        rettypes = set()
-        for blk in utils.itervalues(self.blocks):
-            term = blk.terminator
-            if isinstance(term, ir.Return):
-                rettypes.add(typemap[term.value.name])
-
+    def _unify_return_types(self, rettypes):
         if rettypes:
             unified = self.context.unify_types(*rettypes)
             if unified is None or not unified.is_precise():
@@ -712,8 +810,15 @@ class TypeInferer(object):
                                   % ", ".join(sorted(map(str, rettypes))))
             return unified
         else:
+            # Function without a successful return path
             return types.none
 
+    def get_return_type(self, typemap):
+        rettypes = set()
+        for var in self._get_return_vars():
+            rettypes.add(typemap[var.name])
+        return self._unify_return_types(rettypes)
+
     def get_state_token(self):
         """The algorithm is monotonic.  It can only grow or "refine" the
         typevar map.
@@ -731,6 +836,8 @@ class TypeInferer(object):
             self.typeof_delitem(inst)
         elif isinstance(inst, ir.SetAttr):
             self.typeof_setattr(inst)
+        elif isinstance(inst, ir.Print):
+            self.typeof_print(inst)
         elif isinstance(inst, (ir.Jump, ir.Branch, ir.Return, ir.Del)):
             pass
         elif isinstance(inst, ir.StaticRaise):
@@ -764,6 +871,12 @@ class TypeInferer(object):
         self.constraints.append(constraint)
         self.calls.append((inst, constraint))
 
+    def typeof_print(self, inst):
+        constraint = PrintConstraint(args=inst.args, vararg=inst.vararg,
+                                     loc=inst.loc)
+        self.constraints.append(constraint)
+        self.calls.append((inst, constraint))
+
     def typeof_assign(self, inst):
         value = inst.value
         if isinstance(value, ir.Const):
@@ -801,18 +914,19 @@ class TypeInferer(object):
                                               loc=inst.loc))
 
     def typeof_const(self, inst, target, const):
-        self.lock_type(target.name, self.resolve_value_type(inst, const))
+        self.lock_type(target.name, self.resolve_value_type(inst, const),
+                       loc=inst.loc)
 
     def typeof_yield(self, inst, target, yield_):
         # Sending values into generators isn't supported.
-        self.add_type(target.name, types.none)
+        self.add_type(target.name, types.none, loc=inst.loc)
 
     def sentry_modified_builtin(self, inst, gvar):
         """
         Ensure that builtins are not modified.
         """
         if (gvar.name in ('range', 'xrange') and
-                    gvar.value not in utils.RANGE_ITER_OBJECTS):
+            gvar.value not in utils.RANGE_ITER_OBJECTS):
             bad = True
         elif gvar.name == 'slice' and gvar.value is not slice:
             bad = True
@@ -825,8 +939,57 @@ class TypeInferer(object):
             raise TypingError("Modified builtin '%s'" % gvar.name,
                               loc=inst.loc)
 
+    def resolve_call(self, fnty, pos_args, kw_args):
+        """
+        Resolve a call to a given function type.  A signature is returned.
+        """
+        if isinstance(fnty, types.RecursiveCall):
+            # Self-recursive call
+            disp = fnty.dispatcher_type.dispatcher
+            pysig, args = disp.fold_argument_types(pos_args, kw_args)
+
+            # Fetch the return type as given by the user
+            rettypes = set()
+            for retvar in self._get_return_vars():
+                typevar = self.typevars[retvar.name]
+                if not typevar.defined:
+                    raise TypeError("recursive calls need an explicit signature in jit()")
+                rettypes.add(typevar.getone())
+            return_type = self._unify_return_types(rettypes)
+
+            # Match call arguments with current inference arguments
+            assert len(args) == len(self.arg_names)
+            formal_args = [self.typevars[self.arg_names[i]].getone()
+                           for i in range(len(args))]
+            for formal_ty, actual_ty in zip(formal_args, args):
+                if not self.context.can_convert(actual_ty, formal_ty):
+                    raise TypeError("bad self-recursive call with argument types %s" % (args,))
+
+            sig = typing.signature(return_type, *formal_args)
+            sig.pysig = pysig
+            return sig
+        else:
+            # Normal non-recursive call
+            return self.context.resolve_function_type(fnty, pos_args, kw_args)
+
     def typeof_global(self, inst, target, gvar):
         typ = self.context.resolve_value_type(gvar.value)
+
+        if (typ is None and gvar.name == self.py_func.__name__
+            and gvar.name in _temporary_dispatcher_map):
+            # Self-recursion case where the dispatcher is not (yet?) known
+            # as a global variable
+            typ = types.Dispatcher(_temporary_dispatcher_map[gvar.name])
+
+        if isinstance(typ, types.Dispatcher) and typ.dispatcher.is_compiling:
+            # Recursive call
+            if typ.dispatcher.py_func is self.py_func:
+                typ = types.RecursiveCall(typ)
+            else:
+                raise NotImplementedError(
+                    "call to %s: mutual recursion not supported"
+                    % typ.dispatcher)
+
         if isinstance(typ, types.Array):
             # Global array in nopython mode is constant
             # XXX why layout='C'?
@@ -834,7 +997,7 @@ class TypeInferer(object):
 
         if typ is not None:
             self.sentry_modified_builtin(inst, gvar)
-            self.lock_type(target.name, typ)
+            self.lock_type(target.name, typ, loc=inst.loc)
             self.assumed_immutables.add(inst)
         else:
             raise TypingError("Untyped global name '%s'" % gvar.name,
@@ -844,7 +1007,7 @@ class TypeInferer(object):
         if expr.op == 'call':
             if isinstance(expr.func, ir.Intrinsic):
                 sig = expr.func.type
-                self.add_type(target.name, sig.return_type)
+                self.add_type(target.name, sig.return_type, loc=inst.loc)
                 self.add_calltype(expr, sig)
             else:
                 self.typeof_call(inst, target, expr)
