@@ -29,9 +29,11 @@ class PrintItem(AbstractTemplate):
     key = "print_item"
 
     def is_accepted_type(self, ty):
-        if isinstance(ty, (types.Integer, types.Boolean, types.Float,
-                           types.CharSeq)):
-            return True
+        if isinstance(ty, (types.Array, types.Record)):
+            # These types require an environment to serialize back to a
+            # Python object.
+            return False
+        return True
 
     def generic(self, args, kws):
         arg, = args
